-- a few helpers for arrays that never modify their inputs

function array_join(a, b)
  local result = {}
  for _,v in ipairs(a) do table.insert(result, v) end
  for _,v in ipairs(b) do table.insert(result, v) end
  return result
end

function map(arr, f)
  local result = {}
  for _,v in ipairs(arr) do table.insert(result, f(v)) end
  return result
end

function reduce(arr, init, f)
  local result = init
  for _,v in ipairs(arr) do
    result = f(result, v)
  end
  return result
end

function sum(arr, f)
  return reduce(map(arr, f), 0,
    function(a, b) return a+b end)
end

function product(arr, f)
  return reduce(map(arr, f), 1,
    function(a, b) return a*b end)
end

function keep(arr, f)
  local result = {}
  for _,v in ipairs(arr) do
    if f(v) then table.insert(result, v) end end
  return result
end

function remove(arr, f)
  local result = {}
  for _,v in ipairs(arr) do
    if not f(v) then table.insert(result, v) end end
  return result
end

function find(arr, f)
  for _,v in ipairs(arr) do
    if f(v) then return v end
  end
end
